@@ -5,13 +5,29 @@ from scipy.signal import savgol_filter
 from .utils import lowpass_butter, find_peaks
 from .move import rotate_matrix
 
-def auto_process(data, wheelsize=0.31, rimsize=0.27, sfreq=200, co_f=15, ord_f=2, co_s=6, ord_s=2, force=True,
-                 speed=True, variable="torque", cutoff=0.0, wl=201, ord_a=2, minpeak=5.0):
+
+def auto_process(
+    data,
+    wheelsize=0.31,
+    rimsize=0.27,
+    sfreq=200,
+    co_f=15,
+    ord_f=2,
+    co_s=6,
+    ord_s=2,
+    force=True,
+    speed=True,
+    variable="torque",
+    cutoff=0.0,
+    wl=201,
+    ord_a=2,
+    minpeak=5.0,
+):
     """
     Top level processing function that performs all processing steps for mw/ergo data.
 
     Contains all signal processing steps in fixed order. It is advised to use this function for all (pre-)processing.
-    If needed take a look at a specific function to see how it works.
+    If needed to take a look at a specific function to see how it works.
 
     Parameters
     ----------
@@ -39,7 +55,7 @@ def auto_process(data, wheelsize=0.31, rimsize=0.27, sfreq=200, co_f=15, ord_f=2
         variable name used for peak (push) detection
     cutoff : float
         noise level for peak (push) detection
-    wl : float
+    wl : int
         window length angle filter
     ord_a : int
         order angle filter [..]
@@ -67,7 +83,7 @@ def auto_process(data, wheelsize=0.31, rimsize=0.27, sfreq=200, co_f=15, ord_f=2
     return data, pushes
 
 
-def filter_mw(data, sfreq=200., co_f=15., ord_f=2, wl=201, ord_a=2, force=True, speed=True):
+def filter_mw(data, sfreq=200.0, co_f=15.0, ord_f=2, wl=201, ord_a=2, force=True, speed=True):
     """
     Filters measurement wheel data.
 
@@ -83,7 +99,7 @@ def filter_mw(data, sfreq=200., co_f=15., ord_f=2, wl=201, ord_a=2, force=True,
         cutoff frequency force filter [Hz]
     ord_f : int
         order force filter [..]
-    wl : float
+    wl : int
         window length angle filter
     ord_a : int
         order angle filter [..]
@@ -111,7 +127,7 @@ def filter_mw(data, sfreq=200., co_f=15., ord_f=2, wl=201, ord_a=2, force=True,
     return data
 
 
-def filter_ergo(data, co_f=15., ord_f=2, co_s=6., ord_s=2, force=True, speed=True):
+def filter_ergo(data, co_f=15.0, ord_f=2, co_s=6.0, ord_s=2, force=True, speed=True):
     """
     Filters ergometer data.
 
@@ -179,7 +195,7 @@ def process_mw(data, wheelsize=0.31, rimsize=0.275, sfreq=200):
     +------------+----------------------+-----------+
     | power      | power                | W         |
     +------------+----------------------+-----------+
-    | work       | instantanious work   | J         |
+    | work       | instantaneous work   | J         |
     +------------+----------------------+-----------+
 
     Parameters
@@ -235,7 +251,7 @@ def process_ergo(data, wheelsize=0.31, rimsize=0.275):
     +------------+----------------------+-----------+
     | power      | power                | W         |
     +------------+----------------------+-----------+
-    | work       | instantanious work   | J         |
+    | work       | instantaneous work   | J         |
     +------------+----------------------+-----------+
     | uforce     | effective force      | N         |
     +------------+----------------------+-----------+
@@ -331,6 +347,10 @@ def push_by_push_mw(data, variable="torque", cutoff=0.0, minpeak=5.0, mindist=5,
         noise level for peak (push) detection
     minpeak : float
         min peak height for peak (push) detection
+    mindist : int
+        minimum sample distance between peak candidates, can be used to speed up algorithm
+    verbose : Boolean
+        can be used to print out the number of pushes, default = True
 
     Returns
     -------
@@ -339,9 +359,36 @@ def push_by_push_mw(data, variable="torque", cutoff=0.0, minpeak=5.0, mindist=5,
     """
     peaks = find_peaks(data[variable], cutoff, minpeak, mindist)
 
-    keys = ["start", "stop", "peak", "tstart", "tstop", "tpeak", "cangle", "cangle_deg", "ptime", "meanpower",
-            "maxpower", "meantorque", "maxtorque", "meanuforce", "maxuforce", "meanforce", "maxforce", "work",
-            "meanfeff", "maxfeff", "meanftot", "maxftot", "slope", "smoothness", "ctime", "reltime", "cwork", "negwork"]
+    keys = [
+        "start",
+        "stop",
+        "peak",
+        "tstart",
+        "tstop",
+        "tpeak",
+        "cangle",
+        "cangle_deg",
+        "ptime",
+        "meanpower",
+        "maxpower",
+        "meantorque",
+        "maxtorque",
+        "meanuforce",
+        "maxuforce",
+        "meanforce",
+        "maxforce",
+        "work",
+        "meanfeff",
+        "maxfeff",
+        "meanftot",
+        "maxftot",
+        "slope",
+        "smoothness",
+        "ctime",
+        "reltime",
+        "cwork",
+        "negwork",
+    ]
     pbp = pd.DataFrame(data=np.full((len(peaks["start"]), len(keys)), np.NaN), columns=keys)  # preallocate dataframe
 
     pbp["start"] = peaks["start"]
@@ -428,7 +475,7 @@ def push_by_push_ergo(data, variable="power", cutoff=0.0, minpeak=50.0, mindist=
     Parameters
     ----------
     data : dict
-        wheelchair ergometer dictionary
+        wheelchair ergometer dictionary  with left, right and mean DataFrame
     variable : str
         variable name used for peak (push) detection, default = power
     cutoff : float
@@ -437,6 +484,8 @@ def push_by_push_ergo(data, variable="power", cutoff=0.0, minpeak=50.0, mindist=
         min peak height for peak (push) detection, default = 50.0
     mindist : int
         minimum sample distance between peak candidates, can be used to speed up algorithm
+    verbose : Boolean
+        can be used to print out the number of pushes for left, right and mean, default = True
 
     Returns
     -------
@@ -444,15 +493,40 @@ def push_by_push_ergo(data, variable="power", cutoff=0.0, minpeak=50.0, mindist=
         dictionary with left, right and mean push-by-push DataFrame
     """
     pbp_sides = {"left": [], "right": [], "mean": []}
-    keys = ["start", "stop", "peak", "tstart", "tstop", "tpeak", "cangle", "cangle_deg", "ptime", "meanpower",
-            "maxpower", "meantorque", "maxtorque", "meanuforce", "maxuforce", "meanforce", "maxforce", "meanspeed",
-            "maxspeed", "work", "slope", "smoothness", "ctime", "reltime", "cwork", "negwork"]
+    keys = [
+        "start",
+        "stop",
+        "peak",
+        "tstart",
+        "tstop",
+        "tpeak",
+        "cangle",
+        "cangle_deg",
+        "ptime",
+        "meanpower",
+        "maxpower",
+        "meantorque",
+        "maxtorque",
+        "meanuforce",
+        "maxuforce",
+        "meanforce",
+        "maxforce",
+        "meanspeed",
+        "maxspeed",
+        "work",
+        "slope",
+        "smoothness",
+        "ctime",
+        "reltime",
+        "cwork",
+        "negwork",
+    ]
 
     for side in data:
-        if (side == 'left') | (side == 'right'):
+        if (side == "left") | (side == "right"):
             peaks = find_peaks(data[side][variable], cutoff, minpeak, mindist)
-        if (side == 'mean'):
-            peaks = find_peaks(data[side][variable], cutoff, (minpeak*2), mindist)
+        else:
+            peaks = find_peaks(data[side][variable], cutoff, (minpeak * 2), mindist)
         pbp = pd.DataFrame(data=np.full((len(peaks["start"]), len(keys)), np.NaN), columns=keys)  # preallocate
 
         pbp["start"] = peaks["start"]
@@ -490,8 +564,11 @@ def push_by_push_ergo(data, variable="power", cutoff=0.0, minpeak=50.0, mindist=
         pbp_sides[side] = pd.DataFrame(pbp)
 
     if verbose:
-        print("\n" + "=" * 80 + f"\nFound left: {len(pbp_sides['left'])} , right: {len(pbp_sides['right'])} and mean: {len(pbp_sides['mean'])} pushes!\n"
-              + "=" * 80 + "\n")
+        print(
+            "\n" + "=" * 80 + f"\nFound left: {len(pbp_sides['left'])} , "
+            f"right: {len(pbp_sides['right'])} and "
+            f"mean: {len(pbp_sides['mean'])} pushes!\n" + "=" * 80 + "\n"
+        )
     return pbp_sides
 
 
@@ -513,11 +590,11 @@ def camber_correct(data, ang):
         corrected for camber angle
     """
 
-    force = data[['fx', 'fy', 'fz']].T
-    torques = data[['mx', 'my', 'torque']].T
+    force = data[["fx", "fy", "fz"]].T
+    torques = data[["mx", "my", "torque"]].T
 
-    rotmat = rotate_matrix(ang/(180*np.pi), axis='x')
-    data[['fx', 'fy', 'fz']] = np.dot(rotmat, force).T
-    data[['mx', 'my', 'torque']] = np.dot(rotmat, torques).T
+    rotmat = rotate_matrix(ang / (180 * np.pi), axis="x")
+    data[["fx", "fy", "fz"]] = np.dot(rotmat, force).T
+    data[["mx", "my", "torque"]] = np.dot(rotmat, torques).T
 
     return data
