@@ -14,7 +14,9 @@ def plot_pushes(data, pushes, var="torque", start=True, stop=True, peak=True, ax
     Parameters
     ----------
     data : pd.DataFrame
+        measurement wheel dataframe or the selected side from the ergometer dict, left, right or mean
     pushes : pd.DataFrame
+        measurement wheel push by push dataframe or the selected side from the ergometer dict, left, right or mean
     var : str
         variable to plot, default is torque
     start : bool
@@ -24,7 +26,7 @@ def plot_pushes(data, pushes, var="torque", start=True, stop=True, peak=True, ax
     peak : bool
         plot push peaks, default is True
     ax : axis object
-        Axis to plot on, you can add your own or it will make a new one.
+        Axis to plot on, you can add your own, or it will make a new one.
 
     Returns
     -------
@@ -76,12 +78,12 @@ def plot_pushes_ergo(data, pushes, title=None, var="power", start=True, stop=Tru
     """
     _, axes = plt.subplots(3, 1, sharex="all", sharey="all")
     if title:
-        plt.suptitle('Push detection: ' + str(title))
+        plt.suptitle("Push detection: " + str(title))
     if not title:
-        plt.suptitle('Push detection')
+        plt.suptitle("Push detection")
     for idx, side in enumerate(data):
         axes[idx] = plot_pushes(data[side], pushes[side], var=var, start=start, stop=stop, peak=peak, ax=axes[idx])
-        axes[idx].set_title(str(side) + ' ' + str(var))
+        axes[idx].set_title(str(side) + " " + str(var))
     plt.tight_layout()
     return axes
 
@@ -100,7 +102,7 @@ def bland_altman_plot(data1, data2, ax=None, condition=None):
     data2 : np.array, pd.Series
         Second variable
     ax : axis object, optional
-        Axis to plot on, you can add your own or it will make a new one.
+        Axis to plot on, you can add your own, or it will make a new one.
     condition : str, optional
         add labels to the plot
 
@@ -120,10 +122,10 @@ def bland_altman_plot(data1, data2, ax=None, condition=None):
 
     with plt.style.context("seaborn-white"):
         ax.scatter(mean, diff)
-        ax.axhline(0, color='dimgray', linestyle='-')
-        ax.axhline(md, color='darkgray', linestyle='--')
-        ax.axhline(md + 1.96 * sd, color='lightcoral', linestyle='--')
-        ax.axhline(md - 1.96 * sd, color='lightcoral', linestyle='--')
+        ax.axhline(0, color="dimgray", linestyle="-")
+        ax.axhline(md, color="darkgray", linestyle="--")
+        ax.axhline(md + 1.96 * sd, color="lightcoral", linestyle="--")
+        ax.axhline(md - 1.96 * sd, color="lightcoral", linestyle="--")
         ax.set_ylim([md - 3 * sd, md + 3 * sd])
         if condition:
             ax.set_xlabel(f"Mean of {condition}")
@@ -132,7 +134,7 @@ def bland_altman_plot(data1, data2, ax=None, condition=None):
     return ax
 
 
-def vel_plot(time, vel, name=''):
+def vel_plot(time, vel, name=""):
     """
     Plot velocity versus time
 
@@ -152,20 +154,20 @@ def vel_plot(time, vel, name=''):
     """
     plt.style.use("seaborn-darkgrid")
     fig, ax = plt.subplots(1, 1, figsize=[10, 6])
-    ax.plot(time, vel, 'r')
+    ax.plot(time, vel, "r")
     ax.set_xlabel("Time [s]", fontsize=12)
     ax.set_ylabel("Velocity [m/s]", fontsize=12)
-    ax.tick_params(axis='y', colors='r', labelsize=12)
-    ax.tick_params(axis='x', labelsize=12)
-    ax.yaxis.label.set_color('r')
+    ax.tick_params(axis="y", colors="r", labelsize=12)
+    ax.tick_params(axis="x", labelsize=12)
+    ax.yaxis.label.set_color("r")
     ax.set_title(f"{name} Velocity")
     ax.set_ylim(0, np.max(vel) + 0.5)
-    ax.autoscale(axis='x', tight=True)
+    ax.autoscale(axis="x", tight=True)
 
     return ax
 
 
-def vel_peak_plot(time, vel, name=''):
+def vel_peak_plot(time, vel, name=""):
     """
     Plot velocity versus time, with vel_peak
 
@@ -190,23 +192,22 @@ def vel_peak_plot(time, vel, name=''):
     # Create time vs. velocity figure with vel_peak
     plt.style.use("seaborn-darkgrid")
     fig, ax = plt.subplots(1, 1, figsize=[10, 6])
-    ax.plot(time, vel, 'r')
-    ax.plot(time[y_max_vel], vel[y_max_vel], 'ko',
-            label='Vel$_{peak}$: ' + str(round(y_max_vel_value, 2)) + ' m/s')
+    ax.plot(time, vel, "r")
+    ax.plot(time[y_max_vel], vel[y_max_vel], "ko", label="Vel$_{peak}$: " + str(round(y_max_vel_value, 2)) + " m/s")
     ax.set_xlabel("Time [s]", fontsize=12)
     ax.set_ylabel("Velocity [m/s]", fontsize=12)
-    ax.tick_params(axis='y', colors='r', labelsize=12)
-    ax.tick_params(axis='x', labelsize=12)
-    ax.yaxis.label.set_color('r')
+    ax.tick_params(axis="y", colors="r", labelsize=12)
+    ax.tick_params(axis="x", labelsize=12)
+    ax.yaxis.label.set_color("r")
     ax.set_title(f"{name} Velocity with vel_peak")
-    ax.legend(loc='lower right', prop={'size': 12})
+    ax.legend(loc="lower right", prop={"size": 12})
     ax.set_ylim(0, y_max_vel_value + 0.5)
-    ax.autoscale(axis='x', tight=True)
+    ax.autoscale(axis="x", tight=True)
 
     return ax
 
 
-def vel_peak_dist_plot(time, vel, dist, name=''):
+def vel_peak_dist_plot(time, vel, dist, name=""):
     """
     Plot velocity and distance against time
 
@@ -234,32 +235,31 @@ def vel_peak_dist_plot(time, vel, dist, name=''):
     plt.style.use("seaborn-darkgrid")
     fig, ax1 = plt.subplots(1, 1, figsize=[10, 6])
     ax1.set_ylim(0, y_max_vel_value + 0.5)
-    ax1.plot(time, vel, 'r')
-    ax1.plot(time[y_max_vel], vel[y_max_vel], 'ko',
-             label='Vel$_{peak}$: ' + str(round(y_max_vel_value, 2)) + ' m/s')
+    ax1.plot(time, vel, "r")
+    ax1.plot(time[y_max_vel], vel[y_max_vel], "ko", label="Vel$_{peak}$: " + str(round(y_max_vel_value, 2)) + " m/s")
     ax1.set_xlabel("Time [s]", fontsize=12)
     ax1.set_ylabel("Velocity [m/s]", fontsize=12)
-    ax1.yaxis.label.set_color('r')
-    ax1.tick_params(axis='y', colors='r', labelsize=12)
-    ax1.tick_params(axis='x', labelsize=12)
+    ax1.yaxis.label.set_color("r")
+    ax1.tick_params(axis="y", colors="r", labelsize=12)
+    ax1.tick_params(axis="x", labelsize=12)
     ax1.set_title(f"{name} Velocity and distance with vel_peak")
-    ax1.legend(loc='lower right', prop={'size': 12})
-    ax1.autoscale(axis='x', tight=True)
+    ax1.legend(loc="lower right", prop={"size": 12})
+    ax1.autoscale(axis="x", tight=True)
 
     # Create time vs. distance figure
     ax2 = ax1.twinx()
-    ax2.set_ylim(0, max(dist)+1)
+    ax2.set_ylim(0, max(dist) + 1)
     ax2.plot(time, dist)
-    ax2.plot(time[y_max_vel], dist[y_max_vel], 'ko')
+    ax2.plot(time[y_max_vel], dist[y_max_vel], "ko")
     ax2.set_ylabel("Distance [m]", fontsize=12)
-    ax2.tick_params(axis='y', colors='b', labelsize=12)
-    ax2.yaxis.label.set_color('b')
-    ax2.autoscale(axis='x', tight=True)
+    ax2.tick_params(axis="y", colors="b", labelsize=12)
+    ax2.yaxis.label.set_color("b")
+    ax2.autoscale(axis="x", tight=True)
 
     return ax1, ax2
 
 
-def acc_plot(time, acc, name=''):
+def acc_plot(time, acc, name=""):
     """
     Plot acceleration versus time
 
@@ -280,20 +280,20 @@ def acc_plot(time, acc, name=''):
     # Create time vs. acceleration figure
     plt.style.use("seaborn-darkgrid")
     fig, ax = plt.subplots(1, 1, figsize=[10, 6])
-    ax.plot(time, acc, 'g')
+    ax.plot(time, acc, "g")
     ax.set_xlabel("Time [s]", fontsize=12)
     ax.set_ylabel("Acceleration [m/$s^2$]", fontsize=12)
-    ax.tick_params(axis='y', colors='g', labelsize=12)
-    ax.tick_params(axis='x', labelsize=12)
-    ax.yaxis.label.set_color('g')
+    ax.tick_params(axis="y", colors="g", labelsize=12)
+    ax.tick_params(axis="x", labelsize=12)
+    ax.yaxis.label.set_color("g")
     ax.set_title(f"{name} Acceleration")
     ax.set_ylim(np.min(acc) - 1, np.max(acc) + 1)
-    ax.autoscale(axis='x', tight=True)
+    ax.autoscale(axis="x", tight=True)
 
     return ax
 
 
-def acc_peak_plot(time, acc, name=''):
+def acc_peak_plot(time, acc, name=""):
     """
     Plot acceleration versus time, with acc_peak
 
@@ -318,23 +318,22 @@ def acc_peak_plot(time, acc, name=''):
     # Create time vs. acceleration figure with acc_peak
     plt.style.use("seaborn-darkgrid")
     fig, ax = plt.subplots(1, 1, figsize=[10, 6])
-    ax.plot(time, acc, 'g')
-    ax.plot(time[y_max_acc], acc[y_max_acc], 'k.',
-            label='Acc$_{peak}$: ' + str(round(y_max_acc_value, 2)) + ' m/$s^2$')
+    ax.plot(time, acc, "g")
+    ax.plot(time[y_max_acc], acc[y_max_acc], "k.", label="Acc$_{peak}$: " + str(round(y_max_acc_value, 2)) + " m/$s^2$")
     ax.set_xlabel("Time [s]", fontsize=12)
     ax.set_ylabel("Acceleration [m/$s^2$]", fontsize=12)
-    ax.tick_params(axis='y', colors='g', labelsize=12)
-    ax.tick_params(axis='x', labelsize=12)
-    ax.yaxis.label.set_color('g')
+    ax.tick_params(axis="y", colors="g", labelsize=12)
+    ax.tick_params(axis="x", labelsize=12)
+    ax.yaxis.label.set_color("g")
     ax.set_title(f"{name} Acceleration with acc_peak")
-    ax.legend(loc='lower center', prop={'size': 12})
+    ax.legend(loc="lower center", prop={"size": 12})
     ax.set_ylim(np.min(acc) - 1, y_max_acc_value + 1)
-    ax.autoscale(axis='x', tight=True)
+    ax.autoscale(axis="x", tight=True)
 
     return ax
 
 
-def acc_peak_dist_plot(time, acc, dist, name=''):
+def acc_peak_dist_plot(time, acc, dist, name=""):
     """
     Plot acceleration and distance versus time, with acc_peak
 
@@ -362,32 +361,33 @@ def acc_peak_dist_plot(time, acc, dist, name=''):
     plt.style.use("seaborn-darkgrid")
     fig, ax1 = plt.subplots(1, 1, figsize=[10, 6])
     ax1.set_ylim(np.min(acc) - 1, y_max_acc_value + 1)
-    ax1.plot(time, acc, 'g')
-    ax1.plot(time[y_max_acc], acc[y_max_acc], 'k.',
-             label='Acc$_{peak}$: ' + str(round(y_max_acc_value, 2)) + ' m/$s^2$')
+    ax1.plot(time, acc, "g")
+    ax1.plot(
+        time[y_max_acc], acc[y_max_acc], "k.", label="Acc$_{peak}$: " + str(round(y_max_acc_value, 2)) + " m/$s^2$"
+    )
     ax1.set_xlabel("Time [s]", fontsize=12)
     ax1.set_ylabel("Acceleration [m/$s^2$]", fontsize=12)
-    ax1.tick_params(axis='y', colors='g', labelsize=12)
-    ax1.tick_params(axis='x', labelsize=12)
-    ax1.yaxis.label.set_color('g')
-    ax1.legend(loc='lower center', prop={'size': 12})
+    ax1.tick_params(axis="y", colors="g", labelsize=12)
+    ax1.tick_params(axis="x", labelsize=12)
+    ax1.yaxis.label.set_color("g")
+    ax1.legend(loc="lower center", prop={"size": 12})
     ax1.set_title(f"{name} Acceleration and distance with acc_peak")
-    ax1.autoscale(axis='x', tight=True)
+    ax1.autoscale(axis="x", tight=True)
 
     # Create time vs. distance figure
     ax2 = ax1.twinx()
     ax2.set_ylim(0, max(dist) + 1)
     ax2.plot(time, dist)
-    ax2.plot(time[y_max_acc], dist[y_max_acc], 'k.')
+    ax2.plot(time[y_max_acc], dist[y_max_acc], "k.")
     ax2.set_ylabel("Distance [m]", fontsize=12)
-    ax2.tick_params(axis='y', colors='b', labelsize=12)
-    ax2.yaxis.label.set_color('b')
-    ax2.autoscale(axis='x', tight=True)
+    ax2.tick_params(axis="y", colors="b", labelsize=12)
+    ax2.yaxis.label.set_color("b")
+    ax2.autoscale(axis="x", tight=True)
 
     return ax1, ax2
 
 
-def rot_vel_plot(time, rot_vel, name=''):
+def rot_vel_plot(time, rot_vel, name=""):
     """
     Plot rotational velocity versus time
 
@@ -408,15 +408,15 @@ def rot_vel_plot(time, rot_vel, name=''):
     # Create time vs. rotational velocity figure
     plt.style.use("seaborn-darkgrid")
     fig, ax = plt.subplots(1, 1, figsize=[10, 6])
-    ax.plot(time, rot_vel, 'b')
+    ax.plot(time, rot_vel, "b")
     ax.set_xlabel("Time [s]", fontsize=12)
     ax.set_ylabel("Rotational velocity [deg/s]", fontsize=12)
-    ax.tick_params(axis='y', colors='b', labelsize=12)
-    ax.tick_params(axis='x', labelsize=12)
-    ax.yaxis.label.set_color('b')
-    ax.set_title(f"{name} Rotational Velociy")
+    ax.tick_params(axis="y", colors="b", labelsize=12)
+    ax.tick_params(axis="x", labelsize=12)
+    ax.yaxis.label.set_color("b")
+    ax.set_title(f"{name} Rotational velocity")
     ax.set_ylim(np.min(rot_vel) - 10, np.max(rot_vel) + 10)
-    ax.autoscale(axis='x', tight=True)
+    ax.autoscale(axis="x", tight=True)
 
     return ax
 
@@ -431,9 +431,7 @@ def set_axes_equal_3d(axes):
         axes containing 3D plotted data
     """
     axes.set_box_aspect([1, 1, 1])
-    limits = np.array([axes.get_xlim3d(),
-                       axes.get_ylim3d(),
-                       axes.get_zlim3d()])
+    limits = np.array([axes.get_xlim3d(), axes.get_ylim3d(), axes.get_zlim3d()])
     origin = np.mean(limits, axis=1)
     radius = 0.5 * np.max(np.abs(limits[:, 1] - limits[:, 0]))
     x, y, z = origin
@@ -442,58 +440,67 @@ def set_axes_equal_3d(axes):
     axes.set_zlim3d([z - radius, z + radius])
 
 
-def imu_push_plot(time, vel, acc_raw, name=''):
+def imu_push_plot(sessiondata, acc_frame=True, name='', dec=False):
     """
     Plot push detection with IMUs
 
     Parameters
     ----------
-    time : dict
-        time structure
-    vel : dict
-        velocity structure
-    acc_raw : dict
-        raw acceleration structure
+    sessiondata : dict or pd.Series
+        processed sessiondata structure or pd.Series with frame data
+    acc_frame: boolean
+        set to True if frame acceleration is used, if False the wheel is used
     name : str
         name of a session
+    dec : boolean
+        set to True if main deceleration should be found
 
     Returns
     -------
     ax: axis object
 
     """
-    # Calculate push detection with function
-    sfreq = 1 / time.diff().mean()
-    push_idx, acc_filt, n_pushes, cycle_time, push_freq = push_imu(acc_raw, sfreq)
+    if type(sessiondata) is dict:
+        sessiondata = sessiondata["frame"]
+    sfreq = int(1 / sessiondata['time'].diff().mean())
 
-    # Calculate processed acceleration from velocity
-    acc = lowpass_butter(np.gradient(vel) * sfreq, sfreq=sfreq, cutoff=20)
+    if acc_frame is True:
+        acc = sessiondata['accelerometer_x']
+    else:
+        acc = lowpass_butter(np.gradient(sessiondata['vel']) * sfreq,
+                             sfreq=sfreq, cutoff=10)
+
+    # Changes signal if the main deceleration values should be found
+    if dec is True:
+        acc -= 1
+
+    push_idx, acc_filt, n_pushes, cycle_time, push_freq = push_imu(acc, sfreq)
 
     # Create time vs. velocity with push detection figure
     plt.style.use("seaborn-darkgrid")
     fig, ax1 = plt.subplots(1, 1, figsize=[10, 6])
-    ax1.set_ylim(-6, 6)
-    ax1.plot(time, vel, 'r')
-    ax1.plot(time[push_idx], vel[push_idx], 'k.', markersize=10)
+    ax1.set_ylim(-2, np.max(sessiondata['vel']) + 0.5)
+    ax1.plot(sessiondata['time'], sessiondata['vel'], 'r')
+    ax1.plot(sessiondata['time'][push_idx],
+             sessiondata['vel'][push_idx], 'k.', markersize=10)
     ax1.set_xlabel("Time [s]", fontsize=12)
     ax1.set_ylabel("Velocity [m/s]", fontsize=12)
     ax1.tick_params(axis='y', colors='r', labelsize=12)
     ax1.tick_params(axis='x', labelsize=12)
     ax1.yaxis.label.set_color('r')
     ax1.set_title(f"{name} Push detection Sprint test")
-    ax1.autoscale(axis='x', tight=True)
 
     # Create time vs. acceleration with push detection figure
     ax2 = ax1.twinx()
-    ax2.set_ylim(-25, 25)
-    ax2.plot(time, acc, 'C7', alpha=0.5)
-    ax2.plot(time, acc_filt, 'b')
-    ax2.plot(time[push_idx], acc_filt[push_idx], 'k.', markersize=10, label="Detected push")
+    ax2.set_ylim(-30, 30)
+    ax2.plot(sessiondata['time'], acc, 'C7', alpha=0.5)
+    ax2.plot(sessiondata['time'], acc_filt, 'b')
+    ax2.plot(sessiondata['time'][push_idx], acc_filt[push_idx], 'k.',
+             markersize=10, label="Detected push")
     ax2.set_ylabel("Acceleration [m/$s^2$]", fontsize=12)
     ax2.tick_params(axis='y', colors='b', labelsize=12)
     ax2.yaxis.label.set_color('b')
-    ax2.legend(frameon=True)
-    ax2.autoscale(axis='x', tight=True)
+    ax2.legend(frameon=True, loc='lower right')
 
     return ax1, ax2
 
@@ -501,7 +508,7 @@ def imu_push_plot(time, vel, acc_raw, name=''):
 def plot_power_speed_dist(data, title="", ylim_power=None, ylim_speed=None, ylim_distance=None):
     """
     Plot power, speed and distance versus time for left (solid line) and
-    right (dotted line) seperately
+    right (dotted line) separately
 
     Figure scales automatically, unless you specify it manually with the ylim_* arguments
 
@@ -550,7 +557,9 @@ def plot_power_speed_dist(data, title="", ylim_power=None, ylim_speed=None, ylim
     host.plot(data["left"]["time"], data["left"]["power"], "forestgreen", label="Power left")
     host.plot(data["right"]["time"], data["right"]["power"], "forestgreen", linestyle="dotted", label="Power right")
     par1.plot(data["left"]["time"], data["left"]["speed"], "firebrick", label="Speed left", alpha=0.7)
-    par1.plot(data["right"]["time"], data["right"]["speed"], "firebrick", linestyle="dotted", label="Speed right", alpha=0.7)
+    par1.plot(
+        data["right"]["time"], data["right"]["speed"], "firebrick", linestyle="dotted", label="Speed right", alpha=0.7
+    )
     par2.plot(data["left"]["time"], data["left"]["dist"], "y", label="Distance left", alpha=0.5)
     par2.plot(data["right"]["time"], data["right"]["dist"], "y", linestyle="dotted", label="Distance right", alpha=0.5)
 
@@ -560,17 +569,17 @@ def plot_power_speed_dist(data, title="", ylim_power=None, ylim_speed=None, ylim
     if ylim_power:
         host.set_ylim(ylim_power[0], ylim_power[1])
     if not ylim_power:
-        host.set_ylim(0., max(max(data["left"]["power"]), max(data["right"]["power"])) * 1.5)
+        host.set_ylim(0.0, max(max(data["left"]["power"]), max(data["right"]["power"])) * 1.5)
 
     if ylim_speed:
         par1.set_ylim(ylim_speed[0], ylim_speed[1])
     if not ylim_speed:
-        par1.set_ylim(0., max(max(data["left"]["speed"]), max(data["right"]["speed"])) * 1.1)
+        par1.set_ylim(0.0, max(max(data["left"]["speed"]), max(data["right"]["speed"])) * 1.1)
 
     if ylim_distance:
         par2.set_ylim(ylim_distance[0], ylim_distance[1])
     if not ylim_distance:
-        par2.set_ylim(0., max(max(data["left"]["dist"]), max(data["right"]["dist"])) * 1.1)
+        par2.set_ylim(0.0, max(max(data["left"]["dist"]), max(data["right"]["dist"])) * 1.1)
 
     host.set_title(title)
     host.set_xlabel("Time [s]")
