@@ -8,9 +8,10 @@ import math
 
 import numpy as np
 import pandas as pd
+import matplotlib.pyplot as plt
 from scipy.interpolate import InterpolatedUnivariateSpline, interp1d
 from scipy.optimize import curve_fit
-from scipy.signal import butter, sosfiltfilt
+from scipy.signal import butter, sosfiltfilt, correlate
 
 
 def pick_file(initialdir=None):
@@ -159,7 +160,7 @@ def make_linear_calibration_spline(calibration_points):
     return spl_line
 
 
-def lowpass_butter(array, sfreq=100., cutoff=20., order=2):
+def lowpass_butter(array, sfreq=100.0, cutoff=20.0, order=2):
     """
     Apply a simple zero-phase low-pass Butterworth filter on an array.
 
@@ -182,7 +183,7 @@ def lowpass_butter(array, sfreq=100., cutoff=20., order=2):
     """
     # noinspection PyTupleAssignmentBalance
     array = np.asarray(array)
-    sos = butter(order, cutoff, fs=sfreq, btype='low', output='sos')
+    sos = butter(order, cutoff, fs=sfreq, btype="low", output="sos")
     return sosfiltfilt(sos, array)
 
 
@@ -279,7 +280,7 @@ def merge_chars(chars):
         concatenated characters
 
     """
-    return ''.join([char.decode("utf-8") for char in chars])
+    return "".join([char.decode("utf-8") for char in chars])
 
 
 def find_peaks(data, cutoff=1.0, minpeak=5.0, min_dist=5):
@@ -317,7 +318,7 @@ def find_peaks(data, cutoff=1.0, minpeak=5.0, min_dist=5):
             peaks["start"].append(prom - tmp["start"])
     for key, value in peaks.items():
         peaks[key] = np.unique(value)  # remove possible duplicates
-    peaks["peak"] = [np.argmax(data[start:stop + 1]) + start for start, stop in zip(peaks["start"], peaks["stop"])]
+    peaks["peak"] = [np.argmax(data[start: stop + 1]) + start for start, stop in zip(peaks["start"], peaks["stop"])]
     return peaks
 
 
@@ -360,7 +361,7 @@ def nonlinear_fit_coast_down(time, vel, total_weight):
     """
     # try to determine c1 and c2 with curve_fit for non-linear approach
     initial_velocity = vel[0]
-    vel_func = lambda t, c1, c2: coast_down_velocity(t, initial_velocity, c1, c2, total_weight)  # lock variables
+    vel_func = lambda t, c1, c2: coast_down_velocity(t, initial_velocity, c1, c2, total_weight)  # noqa, lock variables
     (coef1, coef2), pcov = curve_fit(vel_func, time, vel, bounds=(0, [1, 20]))  # 'trf' method but 'lm' would also work
     return coef1, coef2
 
@@ -410,7 +411,7 @@ def calc_inertia(weight=0.8, radius=0.295, length=0.675, period=1.0):
         inertia [kgm2]
 
     """
-    return (weight * 9.81 * radius ** 2 * period ** 2) / (4 * np.pi ** 2 * length)
+    return (weight * 9.81 * radius**2 * period**2) / (4 * np.pi**2 * length)
 
 
 def zerocross1d(x, y, indices=False):
@@ -420,7 +421,7 @@ def zerocross1d(x, y, indices=False):
     Find the zero crossing events in a discrete data set. Linear interpolation is used to determine the actual
     locations of the zero crossing between two data points showing a change in sign. Data point which are zero
     are counted in as zero crossings if a sign change occurs across them. Note that the first and last data point will
-    not be considered whether or not they are zero.
+    not be considered whether they are zero.
 
     Parameters
     ----------
@@ -481,8 +482,8 @@ def camel_to_snake(name: str):
         converted_string
     """
 
-    s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
-    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()
+    s1 = re.sub("(.)([A-Z][a-z]+)", r"\1_\2", name)
+    return re.sub("([a-z0-9])([A-Z])", r"\1_\2", s1).lower()
 
 
 def find_nearest(array, value, index=False):
@@ -496,12 +497,12 @@ def find_nearest(array, value, index=False):
     value : float
         value that you are looking for
     index : bool
-        whether or not you want the index
+        whether you want the index
 
     Returns
     -------
     np.array
-        value or index of nearest value
+        value or index of the nearest value
 
     """
     array = np.asarray(array)
@@ -534,7 +535,7 @@ def split_dataframe(df, inds):
 
 def binned_stats(array, bins=10, pad=True, func=np.mean, nan_func=np.nanmean):
     """
-    Apply a compatible Numpy function to every bins samples (e.g. mean or std).
+    Apply a compatible Numpy function to every bin samples (e.g. mean or std).
 
     Parameters
     ----------
@@ -543,7 +544,7 @@ def binned_stats(array, bins=10, pad=True, func=np.mean, nan_func=np.nanmean):
     bins : int
         number of samples to be averaged
     pad : bool
-        whether or not to pad the array with NaNs if needed
+        whether to pad the array with NaNs if needed
     func
         function that is used when no padding is applied
     nan_func
@@ -552,15 +553,15 @@ def binned_stats(array, bins=10, pad=True, func=np.mean, nan_func=np.nanmean):
     Returns
     -------
     means: np.array
-        array with the mean for every bins samples.
+        array with the mean for every bin samples.
 
     """
     array = np.array(array, dtype=float)  # make sure we have an array
     if pad:
-        array = np.pad(array, (0, bins - array.size % bins), mode='constant', constant_values=np.NaN)
+        array = np.pad(array, (0, bins - array.size % bins), mode="constant", constant_values=np.NaN)
         means = nan_func(array.reshape(-1, bins), axis=1)
     else:
-        means = func(array[:(len(array) // bins) * bins].reshape(-1, bins), axis=1)
+        means = func(array[: (len(array) // bins) * bins].reshape(-1, bins), axis=1)
     return means
 
 
@@ -591,11 +592,12 @@ class Timer:
         print the time between this lap and the previous one
 
     """
+
     timers = dict()
 
     def __init__(self, name="", text="Elapsed time: {:0.4f} seconds", start=True):
         self._start_time = None
-        self._lap_time = 0.
+        self._lap_time = 0.0
         self.name = name
         self.text = text
 
@@ -605,15 +607,13 @@ class Timer:
             self.start()
 
     def start(self):
-        """Start a new timer"""
         if self._start_time is not None:
-            raise TimerError(f"Timer is already running. Use .stop() to stop it")
+            raise TimerError("Timer is already running. Use .stop() to stop it")
         self._start_time = time.perf_counter()
 
     def lap(self, lap_name=""):
-        """Report the elapsed time"""
         if self._start_time is None:
-            raise TimerError(f"Timer is not running. Use .start() to start it")
+            raise TimerError("Timer is not running. Use .start() to start it")
 
         if self._lap_time:
             current_lap = time.perf_counter() - self._lap_time - self._start_time
@@ -627,9 +627,8 @@ class Timer:
         print(self.text.format(current_lap))
 
     def stop(self):
-        """Stop the timer, and report the elapsed time"""
         if self._start_time is None:
-            raise TimerError(f"Timer is not running. Use .start() to start it")
+            raise TimerError("Timer is not running. Use .start() to start it")
 
         elapsed_time = time.perf_counter() - self._start_time
         self._start_time = None
@@ -645,7 +644,7 @@ def power_per_min(data_ergo, dur, start_spiro):
     Calculates the power achieved per min in a graded exercise test, needed for
     the Wasserman plot
 
-    Parameters:
+    Parameters
     ----------
     data_ergo : pd.DataFrame
         processed ergometer data dictionary with dataframes
@@ -657,31 +656,30 @@ def power_per_min(data_ergo, dur, start_spiro):
     Returns
     -------
     power : pd.DataFrame
-        a dataframe containing the mean power output per step, showed as a continu signal
+        a dataframe containing the mean power output per step, showed as a continuous signal
 
 
     """
 
-    n = [*range(math.ceil(dur / 60))]    # define amount of steps
-    mean_power = []     # define mean_power
+    n = [*range(math.ceil(dur / 60))]  # define amount of steps
+    mean_power = []  # define mean_power
+    power = []
 
     for i in n:
-        x = data_ergo['mean']
-        s = x[(x['time'] > ((i+1)*60)-60) & (x['time'] < (((i+1)*60)))]     # cut every step
-        s = s[s['time'] > (s['time'].max()-20)]     # takes last 20s of step
-        mean_p = s['power'].mean()
+        x = data_ergo["mean"]
+        s = x[(x["time"] > ((i + 1) * 60) - 60) & (x["time"] < ((i + 1) * 60))]  # cut every step
+        s = s[s["time"] > (s["time"].max() - 20)]  # takes last 20s of step
+        mean_p = s["power"].mean()
         mean_power.append(mean_p)
 
-    outcome = pd.DataFrame(mean_power, columns=['mean_power'])
+    outcome = pd.DataFrame(mean_power, columns=["mean_power"])
     for i in n:
-        q = [outcome['mean_power'][i]] * 60
-        if i == 0:
-            power = []
+        q = [outcome["mean_power"][i]] * 60
         power = power + q
 
     x = [0] * int(start_spiro)
     power = x + power
-    power = pd.DataFrame(power, columns=['power'])
+    power = pd.DataFrame(power, columns=["power"])
 
     return power
 
@@ -705,3 +703,65 @@ def metamax_to_s(dt):
     h, m, s = dt[:-4].split(":")
     time = int(h) * 3600 + int(m) * 60 + int(s)
     return time
+
+
+def signal_lag(y1, y2, sfreq=100, cutoff=6, order=2, plot=True, verbose=True):
+    """
+    Data alignment function, based on cross-correlation, that can align two devices.
+    Input parameters need to be equal sample frequencies (i.e., a priori interpolation)
+
+    Aligns 2 datasets based on given input variables, after low bandpass filtering.
+    It is advised to use speed data for alignment.
+
+    Parameters
+    ----------
+    y1: np.array, pd.Series
+        Variable from first device which lag of y2 is calculated on, preferably velocity array
+    y2: np.array, pd.Series
+        Variable from second device which is compared to y1, preferably velocity array
+    sfreq : float
+        sample frequency of the signal, default is 100
+    cutoff : float
+        cutoff frequency for the filter, default is 6
+    order : int
+        order of the filter, default is 2
+    plot : boolean
+        can be used to plot the correlation line, default = True
+    verbose : boolean
+        can be used to print out the samples that y2 lags compared to y1, default = True
+
+    Returns
+    -------
+    delay : int
+        the number of samples that y2 should be shifted to correlate maximally to y1
+    maxcorr : int
+        the correlation between y2 and y1, if shifted according to the delay
+
+    See Also
+    --------
+    lowpass_butter
+
+    """
+
+    y1 = lowpass_butter(y1, sfreq=sfreq, cutoff=cutoff, order=order)
+    y2 = lowpass_butter(y2, sfreq=sfreq, cutoff=cutoff, order=order)
+    n = len(y1)
+
+    corr = correlate(y2, y1, mode='same') / np.sqrt(
+        correlate(y1, y1, mode='same')[int(n / 2)] * correlate(y2, y2, mode='same')[int(n / 2)])
+    delay_arr = np.linspace(-0.5 * n, 0.5 * n, n)
+    maxcorr = np.argmax(corr)
+    delay = int(round(delay_arr[maxcorr]))
+
+    if plot:
+        plt.figure()
+        plt.plot(delay_arr, corr)
+        plt.title('Lag: ' + str(delay) + ' s')
+        plt.xlabel('Lag (samples)')
+        plt.ylabel('Correlation coefficient')
+        plt.show()
+
+    if verbose:
+        print("\n" + "=" * 80 + f"\nSignal y2 is {delay} samples off from y1!\n" + "=" * 80 + "\n")
+
+    return delay, maxcorr
